import Logo from '../../assets/pr-logo.png';
import {SiderWidth} from '../../../styles/theme/StyleConstants.js';
import {
  CrownFilled,
  SmileFilled,
  SettingFilled,
  StarTwoTone,
  TableOutlined,
  BarChartOutlined,
  LineChartOutlined,
  PieChartOutlined,
  FundFilled,
  DropboxCircleFilled,
  ProductFilled
} from '@ant-design/icons';

export const ICON_MAPPING = {
  WELCOME:<SmileFilled />,
  ADMIN:<CrownFilled />,
  INDEX:<FundFilled />,
  SETTING:<SettingFilled />,
  STORE:<DropboxCircleFilled />,
  CHAT:<StarTwoTone />,
  MODEL:<ProductFilled />
}

export const DISPLAY_TYPE = {
  TABLE:"table",
  BAR_CHART:"bar_chart",
  LINE_CHART:"line_chart",
  PIE_CHART:"pie_chart",
  AREA_CHART:"area_chart",
}

export const DISPLAY_COND=[
  {
    label: <TableOutlined/>,
    value: DISPLAY_TYPE.TABLE
  },
  {
    label: <BarChartOutlined/>,
    value: DISPLAY_TYPE.BAR_CHART
  },
  // {
  //   label: <LineChartOutlined/>,
  //   value: DISPLAY_TYPE.LINE_CHART
  // },
  // {
  //   label: <PieChartOutlined/>,
  //   value: DISPLAY_TYPE.PIE_CHART
  // }
]

export const router = {
  path: '/',
  routes: [
    {
      path: '/welcome',
      name: '欢迎',
      icon: ICON_MAPPING.WELCOME,
    },
    {
      path: '/modal',
      name: '模型管理',
      icon: ICON_MAPPING.MODEL,
    },
    {
      path: '/index_mana',
      name: '指标管理',
      icon: ICON_MAPPING.INDEX,
      routes: [
        {
          path: 'dimension',
          name: '维度管理',
        },
        {
          path: 'index_df',
          name: '指标定义',
        },
        {
          path: 'index_set',
          name: '指标数据集定义',
        },
      ]
    },
    {
      path: '/store',
      name: '指标市场',
      icon: ICON_MAPPING.STORE,
      routes: [
        {
          path: 'library',
          name: '指标库',
        },
        {
          path: 'data_set',
          name: '指标数据集',
        },
      ]
    },
    {
      path: '/manage',
      name: '系统管理',
      icon: ICON_MAPPING.ADMIN,
      routes: [
        {
          path: 'index',
          name: '指标字典',
          routes: [
            {
              path: 'business',
              name: '业务限定',
            },
            {
              path: 'time_cycle',
              name: '时间周期',
            },
            {
              path: 'db_source',
              name: '数据源管理',
            },
          ]
        },
        {
          path: 'basic',
          name: '授权管理',
          routes: [
            {
              path: 'user',
              name: '用户管理',
            },
            {
              path: 'role',
              name: '角色管理',
            },
          ]
        },
      ]
    }
  ],
}

export const ProCardStyle={
  height: '86.8vh',
  minHeight: 800,
  overflow: 'auto',
}

export const bgLayoutImgList = [
  {
    src: 'https://img.alicdn.com/imgextra/i2/O1CN01O4etvp1DvpFLKfuWq_!!6000000000279-2-tps-609-606.png',
    left: 85,
    bottom: 100,
    height: '303px',
  },
  {
    src: 'https://img.alicdn.com/imgextra/i2/O1CN01O4etvp1DvpFLKfuWq_!!6000000000279-2-tps-609-606.png',
    bottom: -68,
    right: -45,
    height: '303px',
  },
  {
    src: 'https://img.alicdn.com/imgextra/i3/O1CN018NxReL1shX85Yz6Cx_!!6000000005798-2-tps-884-496.png',
    bottom: 0,
    left: 0,
    width: '331px',
  },
];

export const defaultProps = {
  logo:Logo,
  appList: [
    {
      icon: 'https://gw.alipayobjects.com/zos/rmsportal/KDpgvguMpGfqaHPjicRK.svg',
      title: 'Ant Design',
      desc: '杭州市较知名的 UI 设计语言',
      url: 'https://ant.design',
    },
    {
      icon: 'https://gw.alipayobjects.com/zos/antfincdn/FLrTNDvlna/antv.png',
      title: 'AntV',
      desc: '蚂蚁集团全新一代数据可视化解决方案',
      url: 'https://antv.vision/',
      target: '_blank',
    },
    {
      icon: 'https://gw.alipayobjects.com/zos/antfincdn/upvrAjAPQX/Logo_Tech%252520UI.svg',
      title: 'Pro Components',
      desc: '专业级 UI 组件库',
      url: 'https://procomponents.ant.design/',
    },
    {
      icon: 'https://img.alicdn.com/tfs/TB1zomHwxv1gK0jSZFFXXb0sXXa-200-200.png',
      title: 'umi',
      desc: '插件化的企业级前端应用框架。',
      url: 'https://umijs.org/zh-CN/docs',
    },

    {
      icon: 'https://gw.alipayobjects.com/zos/bmw-prod/8a74c1d3-16f3-4719-be63-15e467a68a24/km0cv8vn_w500_h500.png',
      title: 'qiankun',
      desc: '可能是你见过最完善的微前端解决方案🧐',
      url: 'https://qiankun.umijs.org/',
    },
    {
      icon: 'https://gw.alipayobjects.com/zos/rmsportal/XuVpGqBFxXplzvLjJBZB.svg',
      title: '语雀',
      desc: '知识创作与分享工具',
      url: 'https://www.yuque.com/',
    },
    {
      icon: 'https://gw.alipayobjects.com/zos/rmsportal/LFooOLwmxGLsltmUjTAP.svg',
      title: 'Kitchen ',
      desc: 'Sketch 工具集',
      url: 'https://kitchen.alipay.com/',
    },
    {
      icon: 'https://gw.alipayobjects.com/zos/bmw-prod/d3e3eb39-1cd7-4aa5-827c-877deced6b7e/lalxt4g3_w256_h256.png',
      title: 'dumi',
      desc: '为组件开发场景而生的文档工具',
      url: 'https://d.umijs.org/zh-CN',
    },
  ],
  siderWidth:SiderWidth,
  bgLayoutImgList:bgLayoutImgList,
  menu:{
    type: 'group',
    collapsedShowTitle: true,
  },
  breadcrumbRender:false
};

export const mockMessages = {
  "新手指南":{
    path:"/main/welcome",
    msg:"你可以通过[主页](http://localhost:5173/main)向导进行了解本软件，是否跳转到相关页面学习？"
  },
  "用户":{
    path:"/main/manage/basic/user",
    msg:"你可以通过[用户页面](http://localhost:5173/main/manage/basic/user)向导进行用户管理，是否跳转到相关页面学习？"
  },
  "角色":{
    path:"/main/manage/basic/user",
    msg:"你可以通过[角色页面](http://localhost:5173/main/manage/basic/user)向导进行用户管理，是否跳转到相关页面学习？"
  },
  "时间周期":{
    path:"/main/manage/index/time_cycle",
    msg:"你可以通过[时间周期](http://localhost:5173/main/manage/index/time_cycle)向导进行用户管理，是否跳转到相关页面学习？"
  },
};

